# my_api_connector/connector.py
import json
import logging
import aiohttp
from pydantic import BaseModel
from typing import Union, List


class NotAuthenticatedError(Exception):
    pass


class ClientRequest(BaseModel):
    name: str
    expired_date: str = ""


class WGEasyAPIConnector:
    def __init__(self, base_url: str, password: str):
        self.base_url = base_url.rstrip("/")
        self.api_url = f"{self.base_url}/api"
        self.password = password
        self.session = None
        self.cookies = None

    async def __aenter__(self):
        self.session = aiohttp.ClientSession()
        await self.authenticate()
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.session.close()

    async def authenticate(self):
        """Авторизация на сервере, установка cookies"""
        async with self.session.post(f"{self.api_url}/session", data={"password": self.password}) as resp:
            if resp.status != 200:
                raise Exception("Authentication failed")
            self.cookies = resp.cookies

    async def _request(self, method: str, path: str, data: Union[BaseModel, dict, None] = None, params: dict = None):
        url = f"{self.api_url}/{path}"
        headers = {"Content-Type": "application/json"}
        if isinstance(data, BaseModel):
            data = data.model_dump()
        try:
            async with self.session.request(method, url, json=data, params=params, headers=headers,
                                            cookies=self.cookies) as resp:
                text = await resp.text()
                content_type = resp.headers.get("Content-Type", "")
                if "application/json" in content_type:
                    return json.loads(text)
                return text
        except aiohttp.ClientError as e:
            raise Exception("Internal server error")

    async def get_clients(self) -> List[dict]:
        """Получение списка клиентов"""
        try:
            return await self._request("GET", "wireguard/client")
        except Exception as e:
            logging.error(f"Failed to get clients: {str(e)}")
            raise e

    async def create_client(self, client_request: ClientRequest) -> bool:
        """Создание клиента"""
        response = await self._request("POST", "wireguard/client", data=client_request)
        return response.get("success", False)

    async def get_client_config(self, client_id: str) -> str:
        """Получение конфигурации клиента"""
        return await self._request("GET", f"wireguard/client/{client_id}/configuration")

    async def disable_config(self, client_id: str) -> str:
        """Отключение конфигурации клиента"""
        return await self._request("POST", f"wireguard/client/{client_id}/disable")

    async def enable_config(self, client_id: str) -> str:
        """Включение конфигурации клиента"""
        return await self._request("POST", f"wireguard/client/{client_id}/enable")
